var lWarn = {logln:function (o){}};
var lInfo = {logln:function (o){}};

var lInfoFILE_NAME = "led-wizard.log";
lWarn = new Logger(lInfoFILE_NAME);
lInfo = new Logger(lInfoFILE_NAME);

/**
 * Compares version strings
 * @param {string} strV1 version string 1 to compare 
 * @param {string} strV2 version string 1 to compare
 * @returns {Number|undefined} return   number < 0  if strV1 < strV2
 *                                          0       if strV1 = strV2
 *                                      number >0   if strV1 < strV2
 */
function compVersions(strV1, strV2) {
    try {
        var nRes = 0;
        var parts1 = strV1.split('.');
        var parts2 = strV2.split('.');
        var nLen = Math.max(parts1.length, parts2.length);

        for (var i = 0; i < nLen; i++) {
          var nP1 = (i < parts1.length) ? parseInt(parts1[i], 10) : 0;
          var nP2 = (i < parts2.length) ? parseInt(parts2[i], 10) : 0;

          if (isNaN(nP1)) { nP1 = 0; }
          if (isNaN(nP2)) { nP2 = 0; }

          if (nP1 !== nP2) {
            nRes = (nP1 > nP2) ? 1 : -1;
            break;
          }
        }
        return nRes;
    } catch(e) {
        lWarn.logln('Exception in backward-compatibility::compVersions ' + e.name + ' ' + e.description);
        return;       
    }
}

/**
 * Converts dSID to dSUID
 * @param {string} dSID 24 char long string representing hex value
 * @returns {String} returns a 34 char string representing the dSUID
 */
function dSID2dSUID(dSID) {
    try {
        if (dSID.length === 24) {
            //copy the first 6 bytes (12 characters to the new dsuid
            var dSUID = dSID.slice(0, 12);
            //insert 4 bytes of 0
            dSUID += '00000000';
            //append the remaining 6 bytes (12 chars) of the dsid
            dSUID += dSID.slice(12, 24);
            //finaly add 1 byte of 0
            dSUID += '00';
            return dSUID;
        } else {
            return '';
        }
    } catch (e) {
        lWarn.logln('Exception in backward-compatibility::dSID2dSUID ' + e.name + ' ' + e.description);
        return '';
    }
}

/**
 * Converts property tree device entries to dSUID. Stores property tree if anything changed.
 * @returns {undefined}
 */
function convert2DSUID() 
{
    lInfo.logln('backward-compatibility::convert2DSUID');
    var fMigrate=false;
    try {
        var dSUID;
        var deviceNode;
        var dSIDNode;
        var iIndex;
        var basePropertyArray = Property.getNode('devices');
        if (basePropertyArray!==null) {
            var subPropArray = basePropertyArray.getChildren();
            for (iIndex = 0; iIndex < subPropArray.length; iIndex++) {
                deviceNode = subPropArray[iIndex];
                dSIDNode = deviceNode.getChild('dsid');
                if (dSIDNode !== null) {
                    lInfo.logln('--converted to dsuid ' + iIndex);
                    //convert dsid to dsuid
                    dSUID = dSID2dSUID(dSIDNode.getValue());
                    //create a new entry with dsuid as key and fill up with the existing values, resp. dsuid 
                    Property.setProperty('devices/' + dSUID + '/dsuid' , dSUID);
                    Property.setProperty('devices/' + dSUID + '/catalogeID' , deviceNode.getChild('catalogeID').getValue());
                    Property.setProperty('devices/' + dSUID + '/ts' , deviceNode.getChild('ts').getValue());
                    //delete the old entry
                    basePropertyArray.removeChild(dSIDNode.getValue());
                    fMigrate = true;
                }
            }
        }
    } catch (e) {
        lWarn.logln('Exception in backward-compatibility::convert2DSUID ' + e.name + ' ' + e.description);        
    }
    if (fMigrate) {
        Property.store();
    }    
}

/**
 * Calculates needed convertion steps according to current property version and performs them. 
 * @returns {undefined}
 */
function convertPropertyTreeEntries()
{
    try {
        Property.load();
        var newVersion = '1.1.0';
        var propVersion = Property.getNode('version');
        if (propVersion) {
            var currentVersion = propVersion.getValue();
            lInfo.logln('check if convertions from version "' + currentVersion + '" to version "' + newVersion + '" are needed');    
            if (compVersions(currentVersion, '1.0.1') < 0) {
                convert2DSUID();   
            }
        } else {
            convert2DSUID();
        }
    	if (Property.getNode('/scripts/led-wizard/dbcache')!=null) {
    		lInfo.logln('removing old cache');
    		Property.getNode('/scripts/led-wizard').removeChild('dbcache');
    	}
        Property.setProperty('version', newVersion);
        Property.setFlag('version', 'ARCHIVE', true);
        Property.store();
        lInfo.logln("led-wizard running (Version " + newVersion + ")");
    } catch (e) {
        lWarn.logln('Exception in backward-compatibility::convertPropertyTreeEntries ' + e.name + ' ' + e.description);
        Property.store();
    }
}

function prepareStartUpSubscription() {
	lInfo.logln('prepareStartUpSubscription');
	if (Property.getNode('db/manufacture')==null) {
		lWarn.logln('no manufacture loaded yet');
		return;
	}
	try {
		var sManufacture=Property.getProperty('db/manufacture');
		var oManufacture=JSON.parse(sManufacture);
		for (var iIndex=0;iIndex<oManufacture.length;iIndex++) {
			lInfo.logln('make subscription for ' + oManufacture[iIndex].id);
			if (Property.getProperty('subscriptions/' + oManufacture[iIndex].id)==null) {
				var oSubscripion=new Subscription('led-wizard.data-segmented' + oManufacture[iIndex].id,'javascript',
						{'filename1':'/usr/share/dss/data/scripts/http.js',
					     'filename2':'/usr/share/dss/add-ons/led-wizard/fetchSegmentedData.js','script_id':'led-wizard.manufacture-' + oManufacture[iIndex].id});
				oSubscripion.subscribe();
				
				var oSubscripion=new Subscription('led-wizard.query-'+ oManufacture[iIndex].id,'javascript',
						{'filename1':'/usr/share/dss/data/scripts/http.js',
					     'filename2':'/usr/share/dss/add-ons/led-wizard/query.js','script_id':'led-wizard.manufacture-' + oManufacture[iIndex].id});
				oSubscripion.subscribe();
				
				Property.setProperty('subscriptions/' + oManufacture[iIndex].id,true);
				new Event('led-wizard.query-'+ oManufacture[iIndex].id,{actions:'loadProperty',supplierID:oManufacture[iIndex].id}).raise();
			}
		}
	} catch (e) {
		lWarn.logln('error in prepareStartupSubscriptions ' + e);
		
	}
}



if (raisedEvent.name == 'running') {
    convertPropertyTreeEntries();
    prepareStartUpSubscription();
    Property.setProperty('demo/state',false);
} else {
    lWarn.logln('backward-compatibility invoked by unexpected event' + raisedEvent.name);    
}